@@ -2,7 +2,7 @@
 
     This file is part of the Maude 3 interpreter.
 
-    Copyright 1997-2021 SRI International, Menlo Park, CA 94025, USA.
+    Copyright 1997-2023 SRI International, Menlo Park, CA 94025, USA.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -288,14 +288,14 @@ MetaLevel::downModule(DagNode* metaModule)
 				  m->checkFreshVariableNames();
 				  cache.insert(metaModule, m);
 				  //
-				  //	We may have displace a module from the
-				  //	metamodule cache generating garbage in
-				  //	the expression. Also there may be an
-				  //	accumulation of garbage anyway from meta-meta
-				  //	processing so we should tidy the (expression)
-				  //	module cache regularly.
+				  //	We may have displaced a module from the metamodule cache.
+				  //	The displaced module may have had parameter and imports
+				  //	constructed for it which could now be orphans. Now that
+				  //	any parameters and imports of this new module have been
+				  //	processed, the module system is quiescent and we can
+				  //	purge any orphans.
 				  //
-				  owner->destructUnusedModules();
+				  owner->cleanCaches();
 				  return m;
 				}
 			    }
@@ -315,12 +315,11 @@ MetaLevel::downModule(DagNode* metaModule)
       //
       m->deepSelfDestruct();
       //
-      //	Pulling down module expressions may have resulted in
-      //	the creation of cached modules that no longer have
-      //	dependents now that we failed to build the metamodule.
-      //	Thus we now need to tidy the module cache.
+      //	Processing module expressions for parameters and imports may have resulted in the
+      //	construction of modules and views that are now orphaned because we failed to build
+      //	the metamodule. We purge these now.
       //	
-      owner->destructUnusedModules();
+      owner->cleanCaches();
     }
   return 0;
 }
@@ -353,6 +352,8 @@ MetaLevel::downImport(DagNode* metaImport, MetaModule* m)
     mode = ImportModule::EXTENDING;
   else if (mi == includingSymbol)
     mode = ImportModule::INCLUDING;
+  else if (mi == generatedBySymbol)
+    mode = ImportModule::GENERATED_BY;
   else
     return false;
   
@@ -475,6 +476,33 @@ MetaLevel::downQidSet(DagNode* metaQidSet, Vector<int>& ids)
   return true;
 }
 
+bool
+MetaLevel::downConcealedSet(DagNode* metaQidSet, PrintSettings& printSettings)
+{
+  //
+  //	Also sets PRINT_CONCEAL flag if set is non-emty.
+  //
+  Symbol* mq =  metaQidSet->symbol();
+  int id;
+  if (mq == qidSetSymbol)
+    {
+      for (DagArgumentIterator i(metaQidSet); i.valid(); i.next())
+	{
+	  if (!downQid(i.argument(), id))
+	    return false;
+	  printSettings.insertConcealed(id);
+	}
+    }
+  else if (mq == emptyQidSetSymbol)
+    return true;
+  else if (downQid(metaQidSet, id))
+    printSettings.insertConcealed(id);
+  else
+    return false;
+  printSettings.setPrintFlag(PrintSettings::PRINT_CONCEAL);
+  return true;
+}
+
 bool
 MetaLevel::downTypeList(DagNode* metaTypeList, MixfixModule* m, Vector<Sort*>& typeList)
 {
@@ -911,6 +939,7 @@ MetaLevel::downMembAx(DagNode* metaMembAx, MixfixModule* m)
 		  if (ai.flags.getFlag(NONEXEC))
 		    mb->setNonexec();
 		  m->insertSortConstraint(mb);
+		  m->checkSortConstraint(mb);
 		  if (ai.metadata != NONE)
 		    m->insertMetadata(MixfixModule::MEMB_AX, mb, ai.metadata);
 		  if (ai.flags.getFlag(PRINT))
@@ -970,6 +999,7 @@ MetaLevel::downEquation(DagNode* metaEquation, MixfixModule* m)
 			IssueAdvisory("variant attribute not allowed for conditional equation in meta-module " << QUOTE(m) << '.');
 		    }
 		  m->insertEquation(eq);
+		  m->checkEquation(eq);
 		  if (ai.metadata != NONE)
 		    m->insertMetadata(MixfixModule::EQUATION, eq, ai.metadata);
 		  if (ai.flags.getFlag(PRINT))
@@ -1029,6 +1059,7 @@ MetaLevel::downRule(DagNode* metaRule, MixfixModule* m)
 			IssueAdvisory("narrowing attribute not allowed for conditional rule in meta-module " << QUOTE(m) << '.');
 		    }
 		  m->insertRule(rl);
+		  m->checkRule(rl);
 		  if (ai.metadata != NONE)
 		    m->insertMetadata(MixfixModule::RULE, rl, ai.metadata);
 		  if (ai.flags.getFlag(PRINT))
@@ -1240,7 +1271,8 @@ MetaLevel::downTerm(DagNode* metaTerm, MixfixModule* m)
   if (mt == metaTermSymbol)
     {
       FreeDagNode* f = static_cast<FreeDagNode*>(metaTerm);
-      Vector<Term*> argList(0, nrPreallocatedArgs);  // pre-allocate memory for speed
+      Vector<Term*> argList;
+      argList.reserve(nrPreallocatedArgs);  // pre-allocate memory for speed
       if (downOpName(f->getArgument(0), id) && downTermList(f->getArgument(1), m, argList))
 	{
 	  int nrArgs = argList.length();
@@ -1485,39 +1517,40 @@ MetaLevel::downAssignment(DagNode* metaAssignment,
 }
 
 bool
-MetaLevel::downPrintOptionSet(DagNode* metaPrintOptionSet, int& printFlags) const
+MetaLevel::downPrintOptionSet(DagNode* metaPrintOptionSet, PrintSettings& printSettings) const
 {
-  printFlags = 0;
   Symbol* mp = metaPrintOptionSet->symbol();
   if (mp == printOptionSetSymbol)
     {
       for (DagArgumentIterator i(metaPrintOptionSet); i.valid(); i.next())
 	{
-	  if (!downPrintOption(i.argument(), printFlags))
+	  if (!downPrintOption(i.argument(), printSettings))
 	    return false;
 	}
     }
   else if (mp != emptyPrintOptionSetSymbol)
-    return downPrintOption(metaPrintOptionSet, printFlags);
+    return downPrintOption(metaPrintOptionSet, printSettings);
   return true;
 }
 
 bool
-MetaLevel::downPrintOption(DagNode* metaPrintOption, int& printFlags) const
+MetaLevel::downPrintOption(DagNode* metaPrintOption, PrintSettings& printSettings) const
 {
   Symbol* mp = metaPrintOption->symbol();
   if (mp == mixfixSymbol)
-    printFlags |= Interpreter::PRINT_MIXFIX;
+    printSettings.setPrintFlag(PrintSettings::PRINT_MIXFIX);
   else if (mp == withParensSymbol)
-    printFlags |= Interpreter::PRINT_WITH_PARENS;
+    printSettings.setPrintFlag(PrintSettings::PRINT_WITH_PARENS);
+  else if (mp == withSortsSymbol)
+    printSettings.setPrintFlag(PrintSettings::PRINT_DISAMBIG_CONST);
   else if (mp == flatSymbol)
-    printFlags |= Interpreter::PRINT_FLAT;
+    printSettings.setPrintFlag(PrintSettings::PRINT_FLAT);
   else if (mp == formatPrintOptionSymbol)
-    printFlags |= Interpreter::PRINT_FORMAT;
+    printSettings.setPrintFlag(PrintSettings::PRINT_FORMAT);
   else if (mp == numberSymbol)
-    printFlags |= Interpreter::PRINT_NUMBER;
+    printSettings.setPrintFlag(PrintSettings::PRINT_NUMBER);
   else if (mp == ratSymbol)
-    printFlags |= Interpreter::PRINT_RAT;
+    printSettings.setPrintFlag(PrintSettings::PRINT_RAT);
   else
     return false;
   return true;
