@@ -2,7 +2,7 @@
 
     This file is part of the Maude 3 interpreter.
 
-    Copyright 1997-2021 SRI International, Menlo Park, CA 94025, USA.
+    Copyright 1997-2024 SRI International, Menlo Park, CA 94025, USA.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,9 +23,9 @@
 DagNode*
 MetaLevel::upQid(int id, PointerMap& qidMap)
 {
-  void* p = const_cast<void*>(static_cast<const void*>(Token::name(id)));
+  const void* p = Token::name(id);
   DagNode* d = static_cast<DagNode*>(qidMap.getMap(p));
-  if (d == 0)
+  if (d == nullptr)
     {
       d = new QuotedIdentifierDagNode(qidSymbol, Token::backQuoteSpecials(id));
       (void) qidMap.setMap(p, d);
@@ -183,7 +183,8 @@ MetaLevel::upDagNode(DagNode* dagNode,
 		//
 		//	nrArgs is only a suggestion - flattened assoc operators may have more args.
 		//
-		Vector<DagNode*> args2(0, nrArgs);  // can't be static!
+		Vector<DagNode*> args2;  // can't be static!
+		args2.reserve(nrArgs);
 		for (; a.valid(); a.next())
 		  args2.append(upDagNode(a.argument(), m, qidMap, dagNodeMap));
 		args[1] = metaArgSymbol->makeDagNode(args2);
@@ -258,7 +259,7 @@ MetaLevel::upTerm(const Term* term, MixfixModule* m, PointerMap& qidMap)
 		  }
 	      }
 	    args[0] = upQid(id, qidMap);
-	    ArgumentIterator a(*(const_cast<Term*>(term)));
+	    ArgumentIterator a(*term);
 	    if (nrArgs == 1)
 	      args[1] = upTerm(a.argument(), m, qidMap);
 	    else
@@ -266,7 +267,8 @@ MetaLevel::upTerm(const Term* term, MixfixModule* m, PointerMap& qidMap)
 		//
 		//	nrArgs is only a suggestion - flattened assoc operators may have more args.
 		//
-		Vector<DagNode*> args2(0, nrArgs);  // can't be static!
+		Vector<DagNode*> args2;  // can't be static!
+		args2.reserve(nrArgs);
 		for (; a.valid(); a.next())
 		  args2.append(upTerm(a.argument(), m, qidMap));
 		args[1] = metaArgSymbol->makeDagNode(args2);
@@ -329,7 +331,8 @@ MetaLevel::upContext(DagNode* dagNode,
     args[1] = upContext(a.argument(), m, hole, qidMap, dagNodeMap);
   else
     {
-      Vector<DagNode*> args2(0, nrArgs);  // can't be static!
+      Vector<DagNode*> args2;  // can't be static!
+      args2.reserve(nrArgs);
       for (; a.valid(); a.next())
 	args2.append(upContext(a.argument(), m, hole, qidMap, dagNodeMap));
       args[1] = metaArgSymbol->makeDagNode(args2);
@@ -440,8 +443,10 @@ MetaLevel::upDisjointSubstitutions(const Substitution& substitution,
 				   DagNode*& right)
 {
   int nrVariables = variableInfo.getNrRealVariables();
-  Vector<DagNode*> leftArgs(0, nrVariables);
-  Vector<DagNode*> rightArgs(0, nrVariables);
+  Vector<DagNode*> leftArgs;
+  leftArgs.reserve(nrVariables);
+  Vector<DagNode*> rightArgs;
+  rightArgs.reserve(nrVariables);
   for (int i = 0; i < nrVariables; i++)
     {
       VariableTerm* variable = safeCast(VariableTerm*, variableInfo.index2Variable(i));
@@ -805,7 +810,7 @@ MetaLevel::upTypeList(const Vector<Sort*>& types,
   int nrTypes = types.size();
   if (omitLast)
     --nrTypes;
-  if (nrTypes == 0)
+  if (nrTypes <= 0)  // <= rather than == to avoid compiler warning
     return new FreeDagNode(nilQidListSymbol);
   if (nrTypes == 1)
     return upType(types[0], qidMap);
@@ -871,8 +876,10 @@ MetaLevel::upDisjointSubstitutions(const Vector<DagNode*>& unifier,
 				   DagNode*& right)
 {
   int nrVariables = unifier.size();
-  Vector<DagNode*> leftArgs(0, nrVariables);
-  Vector<DagNode*> rightArgs(0, nrVariables);
+  Vector<DagNode*> leftArgs;
+  leftArgs.reserve(nrVariables);
+  Vector<DagNode*> rightArgs;
+  rightArgs.reserve(nrVariables);
   for (int i = 0; i < nrVariables; i++)
     {
       VariableDagNode* variable = variableInfo.index2Variable(i);
