@@ -2,7 +2,7 @@
 
     This file is part of the Maude 3 interpreter.
 
-    Copyright 1997-2020 SRI International, Menlo Park, CA 94025, USA.
+    Copyright 1997-2023 SRI International, Menlo Park, CA 94025, USA.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,6 +24,46 @@
 //	Code for entering stuff into a MixfixModule.
 //
 
+bool
+MixfixModule::unpackLatexMacro(SymbolInfo& si, int arity)
+{
+  Vector<int> unpacked;
+  bool backslashSeen = false;
+  for (const char* p = Token::name(si.latexMacro); *p;)
+    {
+      unsigned char c = *p;
+      if (backslashSeen)
+	backslashSeen = false;
+      else
+	{
+	  if (c == '\\')
+	    backslashSeen = true;
+	  else if (c == '#')
+	    {
+	      ++p;
+	      char c2 = *p;
+	      if (isdigit(c2) && c2 != '0')
+		{
+		  char* endPtr;
+		  int argNr = strtol(p, &endPtr, 10);
+		  if (argNr > 0 && argNr <= arity)
+		    {
+		      unpacked.push_back(- argNr);
+		      p = endPtr;
+		      continue;
+		    }
+		}
+	      --p;
+	      return false;
+	    }
+	}
+      unpacked.push_back(c);
+      ++p;
+    }
+  si.latexMacroUnpacked.swap(unpacked);
+  return true;
+}
+
 Sort*
 MixfixModule::addSort(int name)
 {
@@ -139,10 +179,10 @@ MixfixModule::checkIterated(Symbol* symbol, const Vector<Sort*>& domainAndRange)
       //
       //	At least one symbol aliasing the iterated forms of symbol.
       //
-      FOR_EACH_CONST(j, NumberToSymbolMap, i->second)
+      for (const auto& j : i->second)
 	{
+	  Symbol* pSymbol = j.second;
 	  int overloadType = ADHOC_OVERLOADED;
-	  Symbol* pSymbol = j->second;
 	  const Vector<Sort*>& pDomainAndRange = pSymbol->getOpDeclarations()[0].getDomainAndRange();
 
 	  bool sameRange = (domainAndRange[1]->component() == pDomainAndRange[1]->component());
@@ -182,6 +222,7 @@ MixfixModule::addOpDeclaration(Token prefixName,
 			       int prec,
 			       const Vector<int>& gather,
 			       const Vector<int>& format,
+			       int latexMacro,
 			       int metadata,
 			       bool& firstDecl)
 {
@@ -249,8 +290,7 @@ MixfixModule::addOpDeclaration(Token prefixName,
 			    }
 			  if (metadata != NONE)
 			    insertMetadata(s, s->getOpDeclarations().length(), metadata);
-			  s->addOpDeclaration(domainAndRange,
-					      symbolType.hasFlag(SymbolType::CTOR));
+			  s->addOpDeclaration(domainAndRange, symbolType.hasFlag(SymbolType::CTOR));
 			  return s;
 			}
 		      else
@@ -259,8 +299,7 @@ MixfixModule::addOpDeclaration(Token prefixName,
 				       ": declaration for " << QUOTE(s) <<
 				       " clashes with declaration on " << *s <<
 				       " because of associativity.");
-			  overloadType =
-			    ADHOC_OVERLOADED | DOMAIN_OVERLOADED | RANGE_OVERLOADED;
+			  overloadType = ADHOC_OVERLOADED | DOMAIN_OVERLOADED | RANGE_OVERLOADED;
 			}
 		    }
 		  else
@@ -421,7 +460,13 @@ MixfixModule::addOpDeclaration(Token prefixName,
   symbolInfo.expandBy(1);
   SymbolInfo& si = symbolInfo[nrSymbols];
   int nrUnderscores = Token::extractMixfix(name, si.mixfixSyntax);
-  if (si.mixfixSyntax.length() == 0)
+  if (nrUnderscores == Token::PAREN_MISMATCH)
+    {
+      IssueWarning(LineNumber(prefixName.lineNumber()) << ": mismatched parentheses in operator " <<
+		   QUOTE(prefixName) << ". It will be treated as having prefix syntax only.");
+      symbolType.clearFlags(SymbolType::PREC | SymbolType::GATHER | SymbolType::FORMAT);
+    }
+  else if (si.mixfixSyntax.empty())
     {
       si.prec = 0;
       WarningCheck(!(symbolType.hasFlag(SymbolType::PREC)),
@@ -451,9 +496,8 @@ MixfixModule::addOpDeclaration(Token prefixName,
     {
       if (si.mixfixSyntax.length() == 1 && nrUnderscores == 1)
 	{
-	  IssueWarning(*symbol <<
-		       ": empty syntax not allowed for single argument operators.");
-	  si.mixfixSyntax.contractTo(0);
+	  IssueWarning(*symbol << ": empty syntax not allowed for single argument operators.");
+	  si.mixfixSyntax.clear();
 	  si.prec = 0;
 	  symbolType.clearFlags(SymbolType::PREC | SymbolType::GATHER | SymbolType::FORMAT);
 	}
@@ -463,7 +507,7 @@ MixfixModule::addOpDeclaration(Token prefixName,
 		       ": number of underscores (" << nrUnderscores << ") does not match " <<
 		       "number of arguments (" << nrArgs << ") for operator " <<
 		       QUOTE(symbol) << '.');
-	  si.mixfixSyntax.contractTo(0);
+	  si.mixfixSyntax.clear();
 	  si.prec = 0;
 	  symbolType.clearFlags(SymbolType::PREC | SymbolType::GATHER | SymbolType::FORMAT);
 	}
@@ -487,6 +531,13 @@ MixfixModule::addOpDeclaration(Token prefixName,
 	    }
 	}
     }
+  si.latexMacro = latexMacro;
+  if (symbolType.hasFlag(SymbolType::LATEX) && !unpackLatexMacro(si, nrArgs))
+    {
+      IssueWarning(*symbol << ": bad latex attribute for operator " << QUOTE(symbol) << '.');
+      si.latexMacro = NONE;
+      symbolType.clearFlags(SymbolType::LATEX);
+    }
   si.polymorphIndex = NONE;
   si.symbolType = symbolType;
   si.symbolType.clearFlags(SymbolType::CTOR);  // don't store ctor flag in per-symbol struct
@@ -576,6 +627,11 @@ MixfixModule::addOpDeclaration(Token prefixName,
 		   ": operator " <<
 		   BEGIN_QUOTE << symbol << END_QUOTE <<
 		   " is not allowed to have both object and msg attributes.");
+      WarningCheck(!(symbolType.hasFlag(SymbolType::PORTAL)),
+		   LineNumber(prefixName.lineNumber()) <<
+		   ": operator " <<
+		   BEGIN_QUOTE << symbol << END_QUOTE <<
+		   " is not allowed to have both object and portal attributes.");
 
       if (nrArgs == 0)
 	{
@@ -587,21 +643,26 @@ MixfixModule::addOpDeclaration(Token prefixName,
       else
 	objectSymbols.insert(symbol->getIndexWithinModule());
     }
-  else
+  else if (symbolType.hasFlag(SymbolType::MESSAGE))
     {
-      if (symbolType.hasFlag(SymbolType::MESSAGE))
+      WarningCheck(!(symbolType.hasFlag(SymbolType::PORTAL)),
+		   LineNumber(prefixName.lineNumber()) <<
+		   ": operator " <<
+		   BEGIN_QUOTE << symbol << END_QUOTE <<
+		   " is not allowed to have both msg and portal attributes.");
+
+      if (nrArgs == 0)
 	{
-	  if (nrArgs == 0)
-	    {
-	      IssueWarning(LineNumber(prefixName.lineNumber()) <<
-			   ": msg attribute for operator " <<
-			   BEGIN_QUOTE << symbol << END_QUOTE <<
-			   " which lacks arguments.");
-	    }
-	  else
-	    messageSymbols.insert(symbol->getIndexWithinModule());
+	  IssueWarning(LineNumber(prefixName.lineNumber()) <<
+		       ": msg attribute for operator " <<
+		       BEGIN_QUOTE << symbol << END_QUOTE <<
+		       " which lacks arguments.");
 	}
+      else
+	messageSymbols.insert(symbol->getIndexWithinModule());
     }
+  else if (symbolType.hasFlag(SymbolType::PORTAL))
+    portalSymbols.insert(symbol->getIndexWithinModule());
   return symbol;
 }
 
@@ -623,6 +684,7 @@ MixfixModule::addPolymorph(Token prefixName,
 			   int prec,
 			   const Vector<int>& gather,
 			   const Vector<int>& format,
+			   int latexMacro,
 			   int metadata)
 {
   int index = findPolymorphIndex(prefixName.code(), domainAndRange);
@@ -636,21 +698,6 @@ MixfixModule::addPolymorph(Token prefixName,
     }
   validateAttributes(prefixName, domainAndRange, symbolType);
   int nrArgs = domainAndRange.size() - 1;
-  if (symbolType.hasFlag(SymbolType::ITER))
-    {
-      if (nrArgs != 1)
-	{
-	  IssueWarning(LineNumber(prefixName.lineNumber()) <<
-		       ": declaration for polymorphic operator " << QUOTE(prefixName) <<
-		       " has iter attribute but " << nrArgs << " arguments.");
-	}
-      else if (!(domainAndRange[0] == 0 && domainAndRange[0] == 0))
-	{
-	  IssueWarning(LineNumber(prefixName.lineNumber()) <<
-		       ": declaration for polymorphic operator " << QUOTE(prefixName) <<
-		       " doesn't have both domain and range as polymorphic.");
-	}
-    }
   int nrPolymorphs = polymorphs.length();
   polymorphs.expandBy(1);
   Polymorph& p = polymorphs[nrPolymorphs];
@@ -661,7 +708,13 @@ MixfixModule::addPolymorph(Token prefixName,
   p.identity = 0;
   p.metadata = metadata;
   int nrUnderscores = Token::extractMixfix(prefixName.code(), p.symbolInfo.mixfixSyntax);
-  if (p.symbolInfo.mixfixSyntax.length() == 0)
+  if (nrUnderscores == Token::PAREN_MISMATCH)
+    {
+      IssueWarning(LineNumber(prefixName.lineNumber()) << ": mismatched parentheses in operator " <<
+		   QUOTE(prefixName) << ". It will be treated as having prefix syntax only.");
+      symbolType.clearFlags(SymbolType::PREC | SymbolType::GATHER | SymbolType::FORMAT);
+    }
+  else if (p.symbolInfo.mixfixSyntax.empty())
     {
       p.symbolInfo.prec = 0;
       WarningCheck(!(symbolType.hasFlag(SymbolType::PREC)),
@@ -689,14 +742,14 @@ MixfixModule::addPolymorph(Token prefixName,
     }
   else
     {
-      if (domainAndRange.length() - 1 != nrUnderscores)
+      if (nrArgs != nrUnderscores)
 	{
 	  IssueWarning(LineNumber(prefixName.lineNumber()) <<
 		       ": number of underscores does not match (" << nrUnderscores <<
-		       ") number of arguments (" << domainAndRange.length() - 1 <<
+		       ") number of arguments (" << nrArgs <<
 		       ") for operator " <<
 		       QUOTE(prefixName) << '.');
-	  p.symbolInfo.mixfixSyntax.contractTo(0);
+	  p.symbolInfo.mixfixSyntax.clear();
 	  p.symbolInfo.prec = 0;
 	  symbolType.clearFlags(SymbolType::PREC | SymbolType::GATHER | SymbolType::FORMAT);
 	}
@@ -720,13 +773,12 @@ MixfixModule::addPolymorph(Token prefixName,
 	    }
 	}
     }
+  p.symbolInfo.latexMacro = latexMacro;  // HACK
   p.symbolInfo.polymorphIndex = nrPolymorphs;  // our own index
   p.symbolInfo.symbolType = symbolType;
-  //cout << "symbolType for "<< prefixName << " is " << symbolType << endl;
   p.symbolInfo.next = NONE;
   {
     p.symbolInfo.iflags = ADHOC_OVERLOADED | DOMAIN_OVERLOADED;
-    int nrArgs = domainAndRange.length() - 1;
     for (int i = 0; i < nrArgs; i++)
       {
 	if (domainAndRange[i] == 0)
@@ -807,3 +859,48 @@ MixfixModule::addBubbleSpec(Symbol* topSymbol,
   bubbleComponents.insert(b.componentIndex);
   return nrBubbleSpecs;
 }
+
+void
+MixfixModule::handleSortConstraint(SortConstraint* sortConstraint, bool dnt)
+{
+  if (!dnt && statementTransformer != 0)
+    {
+      StatementTransformer::Outcome result = statementTransformer->transformSortConstraint(sortConstraint);
+      AdvisoryCheck(result != StatementTransformer::Outcome::NOT_TRANSFORMED,
+		    *sortConstraint << ": transformation not performed for:\n  " << sortConstraint);
+    }
+  //
+  //	Any transformation has taken place so its safe to analyze sortConstraint.
+  //
+  checkSortConstraint(sortConstraint);
+}
+
+void
+MixfixModule::handleEquation(Equation* equation, bool dnt)
+{
+  if (!dnt && statementTransformer != 0)
+    {
+      StatementTransformer::Outcome result = statementTransformer->transformEquation(equation);
+      AdvisoryCheck(result != StatementTransformer::Outcome::NOT_TRANSFORMED,
+		    *equation << ": transformation not performed for:\n  " << equation);
+    }
+  //
+  //	Any transformation has taken place so its safe to analyze equation.
+  //
+  checkEquation(equation);
+}
+
+void
+MixfixModule::handleRule(Rule* rule, bool dnt)
+{
+  if (!dnt && statementTransformer != 0)
+    {
+      StatementTransformer::Outcome result = statementTransformer->transformRule(rule);
+      AdvisoryCheck(result != StatementTransformer::Outcome::NOT_TRANSFORMED,
+		    *rule << ": transformation not performed for:\n  " << rule);
+    }
+  //
+  //	Any transformation has taken place so its safe to analyze rule.
+  //
+  checkRule(rule);
+}
