@@ -2,7 +2,7 @@
 
     This file is part of the Maude 3 interpreter.
 
-    Copyright 1997-2021 SRI International, Menlo Park, CA 94025, USA.
+    Copyright 1997-2023 SRI International, Menlo Park, CA 94025, USA.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,6 +24,8 @@
 //      Implementation for class ImportModule.
 //
 
+#include <sstream>
+
 //      utility stuff
 #include "macros.hh"
 #include "vector.hh"
@@ -65,10 +67,13 @@
 #include "strategyExpression.hh"
 
 //	our stuff
+#include "ooSorts.cc"
+#include "pseudoParameters.cc"
 #include "renameModule.cc"
 #include "parameterization.cc"
 #include "instantiateModuleWithFreeParameters.cc"
 #include "instantiateModuleWithBoundParameters.cc"
+#include "latexImportModule.cc"
 
 ImportModule::ImportModule(int name, ModuleType moduleType)
   : MixfixModule(name, moduleType),
@@ -79,6 +84,7 @@ ImportModule::ImportModule(int name, ModuleType moduleType)
   //	object level syntax or meta-syntax, both of which are considered TEXT.
   //
   importPhase = UNVISITED;
+  parameterCopyParameterName = NONE;
   nrSortsFromParameters = 0;
   nrSymbolsFromParameters = 0;
   nrPolymorphsFromParameters = 0;
@@ -98,6 +104,7 @@ ImportModule::ImportModule(int name, ModuleType moduleType, Origin origin, Entit
   //
   addUser(parent);
   importPhase = UNVISITED;
+  parameterCopyParameterName = NONE;
   nrSortsFromParameters = 0;
   nrSymbolsFromParameters = 0;
   nrPolymorphsFromParameters = 0;
@@ -127,8 +134,19 @@ ImportModule::getObjectName() const
 const char*
 ImportModule::importModeString(ImportMode mode)
 {
-  static const char* const modeStrings[] = { "protecting", "extending", "including" };
-  return modeStrings[mode];
+  switch (mode)
+    {
+    case INCLUDING:
+      return "including";
+    case GENERATED_BY:
+      return "generated-by";
+    case EXTENDING:
+      return "extending";
+    case PROTECTING:
+      return "protecting";
+    }
+  CantHappen("bad importation mode" << mode);
+  return 0;
 }
 
 void
@@ -158,6 +176,25 @@ ImportModule::addImport(ImportModule* importedModule,
 		   QUOTE(this) << " not allowed.  Recovering by ignoring import.");
       return;
     }
+  if (!parameterNames.empty())
+    {
+      const EnclosingObject::ParameterSet& pseudos = importedModule->getPseudoParameters();
+      for (int id : parameterNames)
+	{
+	  if (pseudos.find(id) != pseudos.end())
+	    {
+	      IssueWarning(lineNumber << ": importation of " <<
+			   QUOTE(moduleTypeString(t)) << " " << QUOTE(importedModule) << " by " <<
+			   QUOTE(moduleTypeString(getModuleType())) << " " << QUOTE(this) <<
+			   " not allowed because the pseudo-parameter " << QUOTE(Token::name(id)) <<
+			   " of " << QUOTE(moduleTypeString(t)) << " " << QUOTE(importedModule) <<
+			   " would be captured by an actual parameter of " <<
+			   QUOTE(moduleTypeString(getModuleType())) << " " << QUOTE(this) <<
+			   ".");
+	      return;
+	    }
+	}
+    }
   importedModules.append(importedModule);
   importModes.append(mode);
   importedModule->addUser(this);
@@ -347,10 +384,8 @@ ImportModule::importSorts()
   //
   //	We first go through our parameter theories and ask them
   //	to donate their sorts.
-  {
-    FOR_EACH_CONST(i, Vector<ImportModule*>, parameterTheories)
-      (*i)->donateSorts(this);
-  }
+  for (ImportModule* i : parameterTheories)
+    i->donateSorts(this);
   //
   //	We just imported the sorts from our last parameter theory
   //	so record this so we know what sorts came from parameters.
@@ -360,10 +395,8 @@ ImportModule::importSorts()
   //	We now go through our regular imports and ask them to
   //	donate their sorts.
   //
-  {
-    FOR_EACH_CONST(i, Vector<ImportModule*>, importedModules)
-      (*i)->donateSorts(this);
-  }
+  for (ImportModule* i : importedModules)
+    i->donateSorts(this);
   //
   //	Keep track of which sort and subsort declarations were imported
   //	from parameter theories or regular imports.
@@ -389,8 +422,8 @@ ImportModule::donateSorts(ImportModule* importer)
   //	- importer should have identical parameterTheories because our parameter
   //	must be bound.
   //
-  FOR_EACH_CONST(i, Vector<ImportModule*>, importedModules)
-    (*i)->donateSorts(importer);
+  for (ImportModule* i : importedModules)
+    i->donateSorts(importer);
   //
   //	Now handle our sorts.
   //
@@ -406,10 +439,8 @@ ImportModule::importOps()
   //
   //	We first go through our parameter theories and ask them
   //	to donate their operators.
-  {
-    FOR_EACH_CONST(i, Vector<ImportModule*>, parameterTheories)
-      (*i)->donateOps(this);
-  }
+  for (ImportModule* i : parameterTheories)
+    i->donateOps(this);
   //
   //	We just imported the operators from our last parameter theory
   //	so record record the number of symbols and polymorphs we
@@ -421,10 +452,8 @@ ImportModule::importOps()
   //	We now go through our regular imports and ask them to
   //	donate their operators.
   //
-  {
-    FOR_EACH_CONST(i, Vector<ImportModule*>, importedModules)
-      (*i)->donateOps(this);
-  }
+  for (ImportModule* i : importedModules)
+    i->donateOps(this);
   //
   //	Keep track of which operators and operator declarations were
   //	imported from parameter theories or regular imports.
@@ -451,8 +480,8 @@ ImportModule::donateOps(ImportModule* importer)
   //	- importer should have identical parameterTheories because our parameter
   //	must be bound.
   //
-  FOR_EACH_CONST(i, Vector<ImportModule*>, importedModules)
-    (*i)->donateOps(importer);
+  for (ImportModule* i : importedModules)
+    i->donateOps(importer);
   //
   //	Now handle our operators.
   //
@@ -468,18 +497,14 @@ ImportModule::fixUpImportedOps()
   //
   //	We first go through our parameter theories and ask them
   //	to fix up the operators they donated.
-  {
-    FOR_EACH_CONST(i, Vector<ImportModule*>, parameterTheories)
-      (*i)->fixUpDonatedOps(this);
-  }
+  for (ImportModule* i : parameterTheories)
+    i->fixUpDonatedOps(this);
   //
   //	We now go through our regular imports and ask them to
   //	fix up the operators they donated.
   //
-  {
-    FOR_EACH_CONST(i, Vector<ImportModule*>, importedModules)
-      (*i)->fixUpDonatedOps(this);
-  }
+  for (ImportModule* i : importedModules)
+    i->fixUpDonatedOps(this);
 }
 
 void
@@ -494,8 +519,8 @@ ImportModule::fixUpDonatedOps(ImportModule* importer)
   //	- importer should have identical parameterTheories because our parameter
   //	must be bound.
   //
-  FOR_EACH_CONST(i, Vector<ImportModule*>, importedModules)
-    (*i)->fixUpDonatedOps(importer);
+  for (ImportModule* i : importedModules)
+    i->fixUpDonatedOps(importer);
   //
   //	Handle our operators.
   //
@@ -540,18 +565,14 @@ ImportModule::importStatements()
   //	We first go through our parameter theories and ask them
   //	to donate their statements.
   //
-  {
-    FOR_EACH_CONST(i, Vector<ImportModule*>, parameterTheories)
-      (*i)->donateStatements(this);
-  }
+  for (ImportModule* i : parameterTheories)
+    i->donateStatements(this);
   //
   //	We now go through our regular imports and ask them to
   //	donate their statements.
   //
-  {
-    FOR_EACH_CONST(i, Vector<ImportModule*>, importedModules)
-      (*i)->donateStatements(this);
-  }
+  for (ImportModule* i : importedModules)
+    i->donateStatements(this);
 }
 
 void
@@ -571,8 +592,8 @@ ImportModule::donateStatements(ImportModule* importer)
   //	- importer should have identical parameterTheories because our parameter
   //	must be bound.
   //
-  FOR_EACH_CONST(i, Vector<ImportModule*>, importedModules)
-    (*i)->donateStatements(importer);
+  for (ImportModule* i : importedModules)
+    i->donateStatements(importer);
   //
   //	The map from imported module's symbols to importing module's symbols
   //	is built dynamically.
@@ -760,6 +781,7 @@ ImportModule::donateStatements2(ImportModule* importer, ImportTranslation& impor
 	    copy->setNonexec();
 	  copy->setLineNumber(ma->getLineNumber());
 	  importer->insertSortConstraint(copy);
+	  importer->checkSortConstraint(copy);
 	  copyMetadata(importer, importTranslation, MEMB_AX, ma, copy);
 	  //importer->insertMetadata(MEMB_AX, copy, getMetadata(MEMB_AX, ma));
 	}
@@ -785,6 +807,7 @@ ImportModule::donateStatements2(ImportModule* importer, ImportTranslation& impor
 	    copy->setVariant();
 	  copy->setLineNumber(e->getLineNumber());
 	  importer->insertEquation(copy);
+	  importer->checkEquation(copy);
 	  copyMetadata(importer, importTranslation, EQUATION, e, copy);
 	  //importer->insertMetadata(EQUATION, copy, getMetadata(EQUATION, e));
 	}
@@ -810,6 +833,7 @@ ImportModule::donateStatements2(ImportModule* importer, ImportTranslation& impor
 	    copy->setNarrowing();
 	  copy->setLineNumber(r->getLineNumber());
 	  importer->insertRule(copy);
+	  importer->checkRule(copy);
 	  copyMetadata(importer, importTranslation, RULE, r, copy);
 	  //importer->insertMetadata(RULE, copy, getMetadata(RULE, r));
 	}
@@ -862,6 +886,82 @@ ImportModule::resetImportPhase()
     m->resetImportPhase();
 }
 
+void
+ImportModule::printModuleExpression(ostream& s, bool parameterBrackets) const
+{
+  //
+  //	We regenerate the module expression that was responsible for this module in a pretty way.
+  //
+  switch (origin)
+    {
+    case TEXT:
+      {
+	s << Token::name(id());
+	break;
+      }
+    case SUMMATION:
+      {
+	const char* sep = "";
+	for (const ImportModule* i : importedModules)
+	  {
+	    s << sep;
+	    sep = " + ";
+	    i->printModuleExpression(s, parameterBrackets);
+	  }
+	break;
+      }
+    case RENAMING:
+      {
+	if (baseModule->origin == SUMMATION)  // (M + N) * R
+	  s << '(';
+	baseModule->printModuleExpression(s, parameterBrackets);
+	if (baseModule->origin == SUMMATION)
+	  s << ')';
+	s << " * ";
+	canonicalRenaming->printRenaming(s, "(", ", ", true);
+	s << ')';
+	break;
+      }
+    case PARAMETER:
+      {
+	//
+	//	We need to print the name of a parameter copy of a theory like X :: T for show modules.
+	//
+	s << Token::name(parameterCopyParameterName) << " :: ";
+	baseModule->printModuleExpression(s, parameterBrackets);
+	break;
+      }
+    case INSTANTIATION:
+      {
+	if (baseModule->origin == RENAMING)  // (M * R){...}, can't be SUMMATION because of unbound parameters
+	  s << '(';
+	baseModule->printModuleExpression(s, parameterBrackets);
+	if (baseModule->origin == RENAMING)
+	  s << ')';
+	const char* sep = "{";
+	for (const Argument* a : savedArguments)
+	  {
+	    s << sep;
+	    sep = ", ";
+	    if (const View* v = dynamic_cast<const View*>(a))
+	      v->printViewExpression(s, parameterBrackets);  // FIXME
+	    else if (const Parameter* p = dynamic_cast<const Parameter*>(a))
+	      {
+		if (parameterBrackets)
+		  s <<  "[";
+		s << Token::name(p->id());
+		if (parameterBrackets)
+		  s <<  "]";
+	      }
+	  }
+	s << '}';
+	break;
+      }
+    default:
+      CantHappen("bad module origin");
+    }
+}
+
 #ifndef NO_ASSERT
 void
 ImportModule::dumpImports(ostream& s) const
