@@ -2,7 +2,7 @@
 
     This file is part of the Maude 3 interpreter.
 
-    Copyright 1997-2003 SRI International, Menlo Park, CA 94025, USA.
+    Copyright 1997-2023 SRI International, Menlo Park, CA 94025, USA.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -54,7 +54,11 @@ MixfixModule::makeGrammar(bool complexFlag)
   //
   int componentNonTerminalBase = complexFlag ? COMPLEX_BASE : SIMPLE_BASE;
   int nextNonTerminal = componentNonTerminalBase - NUMBER_OF_TYPES * getConnectedComponents().length() + 1;
-  parser = new MixfixParser(*this, complexFlag, componentNonTerminalBase, nextNonTerminal);
+  parser = new MixfixParser(*this,
+			    complexFlag,
+			    componentNonTerminalBase,
+			    nextNonTerminal,
+			    2 * getSymbols().size());  // very rough estimate of number of tokens
   DebugInfo("new parser = " << parser << "  complexFlag = " << complexFlag <<
 	    "  componentNonTerminalBase = " << componentNonTerminalBase <<
 	    "  nextNonTerminal = " << nextNonTerminal);
@@ -88,9 +92,9 @@ MixfixModule::makeGrammar(bool complexFlag)
 void
 MixfixModule::makeParameterizedSortProductions()
 {
-  FOR_EACH_CONST(i, SortMap, sortNames)
+  for (const auto& i : sortNames)
     {
-      int name= i->first;
+      int name= i.first;
       if (Token::auxProperty(name) == Token::AUX_STRUCTURED_SORT)
 	{
 	  //
@@ -115,7 +119,6 @@ MixfixModule::makeParameterizedSortProductions()
 	      p.first->second = nt;
 	      Vector<int> rhs(1);
 	      rhs[0] = t;
-	      //cout << "para " << nt << " ::= " << rhs[0] << endl;
 	      parser->insertProduction(nt, rhs, 0, emptyGather);
 	      parser->insertVariableTerminal(lead, t);
 	    }
@@ -146,6 +149,10 @@ MixfixModule::makeComplexProductions()
   parser->insertProduction(SEARCH_CONNECTIVE, rhs, 0, emptyGather,
 			   MixfixParser::NOP, RewriteSequenceSearch::NORMAL_FORM);
 
+  rhs[0] = arrowHash;
+  parser->insertProduction(SEARCH_CONNECTIVE, rhs, 0, emptyGather,
+			   MixfixParser::NOP, RewriteSequenceSearch::BRANCH);
+
   rhs[0] = MATCH_PAIR;
   parser->insertProduction(MATCH_COMMAND, rhs, 0, gatherAny);
   //
@@ -312,10 +319,10 @@ MixfixModule::makeStrategyLanguageProductions()
 	  {
 	    Vector<int> gather;
 	    //
-	    // Strategies wiht parameters are read in a prefixed form
+	    // Strategies with parameters are read in a prefixed form
 	    //
 	    rhs.append(leftParen);
-	    gather.resize(0);
+	    gather.clear();
 
 	    for (int j = 0; j < nrArgs; j++)
 	      {
@@ -514,7 +521,6 @@ MixfixModule::makeStrategyLanguageProductions()
     parser->insertProduction(STRATEGY_EXPRESSION, rhs, STRAT_TEST_PREC, gatherAny, MixfixParser::MAKE_TEST, UNBOUNDED);
   }
   {
-
     //
     //	<using pair> = <term> using <strategy expression>
     //	<using list> = <using pair> , <using list>
@@ -642,6 +648,9 @@ MixfixModule::makeAttributeProductions()
   rhs[0] = narrowing;
   parser->insertProduction(ATTRIBUTE, rhs, 0, emptyGather,
 			   MixfixParser::MAKE_NARROWING_ATTRIBUTE);
+  rhs[0] = dnt;
+  parser->insertProduction(ATTRIBUTE, rhs, 0, emptyGather,
+			   MixfixParser::MAKE_DNT_ATTRIBUTE);
   //
   //	Print items.
   //
@@ -1130,7 +1139,7 @@ MixfixModule::makeSymbolProductions()
 	  //	<fooTerm> ::= symbolName ( <barTerm> , <bazTerm> , ... , <quuxTerm> )
 	  //
 	  rhs.append(leftParen);
-	  gather.resize(0);
+	  gather.clear();
 	  for (int j = 0; j < nrArgs; j++)
 	    {
 	      gather.append(PREFIX_GATHER);
@@ -1206,6 +1215,30 @@ MixfixModule::makeSymbolProductions()
 		rhs[j] = t;
 	    }
 	  parser->insertProduction(rangeNt, rhs, si.prec, si.gather, MixfixParser::MAKE_TERM, i);
+	  //
+	  //	Special syntax for objects with empty attribute set.
+	  //
+	  if (si.symbolType.getBasicType() == SymbolType::OBJECT_CONSTRUCTOR_SYMBOL)  // has the id-hook
+	    {
+	      Symbol* attributeSetSymbol = safeCast(ObjectConstructorSymbol*, symbol)->getAttributeSetSymbol();
+	      if (attributeSetSymbol)  // has the op-hook
+		{
+		  SymbolType st =  getSymbolType(attributeSetSymbol);
+		  if (st.hasAllFlags(SymbolType::ASSOC | SymbolType::COMM | SymbolType::LEFT_ID | SymbolType::RIGHT_ID))  // correct attributes
+		    {
+		      int lastUnderscoreIndex = nrItems - 1;
+		      while (si.mixfixSyntax[lastUnderscoreIndex] != underscore)
+			--lastUnderscoreIndex;
+		      for (int j = lastUnderscoreIndex + 1; j < nrItems; ++j)
+			rhs[j - 1] = si.mixfixSyntax[j];
+		      rhs.resize(nrItems - 1);
+		      Vector<int> gather(2);
+		      gather[0] = si.gather[0];
+		      gather[1] = si.gather[1];
+		      parser->insertProduction(rangeNt, rhs, si.prec, gather, MixfixParser::MAKE_OBJECT_WITH_EMPTY_ATTRIBUTE_SET, i);
+		    }
+		}
+	    }
 	}
     }
 }
@@ -1266,9 +1299,8 @@ MixfixModule::makeLabelProductions()
 #endif
 
   static Vector<int> rhs(1);
-  FOR_EACH_CONST(i, set<int>, potentialLabels)
+  for (int label : potentialLabels)
     {
-      int label = *i;
       rhs[0] = label;
       parser->insertProduction(LABEL, rhs, 0, emptyGather, MixfixParser::MAKE_LABEL, label);
     }
@@ -1285,11 +1317,9 @@ MixfixModule::makeSpecialProductions()
   //
   //	Productions for existing tokens that have special properties.
   //
-  const IntSet& tokens = parser->getTokenSet();  // HACK
-  int nrTokens = tokens.cardinality();
-  for (int i = 0; i < nrTokens; i++)
-    { 
-      int code = tokens.index2Int(i);
+  const MixfixParser::TokenSet& tokenSet = parser->getTokenSet();  // HACK
+  for (auto code : tokenSet)
+    {
       switch (Token::specialProperty(code))
 	{
 	case Token::SMALL_NAT:
@@ -1428,24 +1458,21 @@ MixfixModule::makeSpecialProductions()
     rhs[0] = t;
     parser->insertProduction(ENDS_IN_COLON_NT, rhs, 0, emptyGather);
   }
-  {
-    FOR_EACH_CONST(i, IntMap, iterSymbols)
-      {
-	//
-	//	For each iter symbol f we create a special terminal
-	//	[ f ] which represents f^n for any positive interger n.
-	//
-	int iterSymbolNameCode = i->first;
-	string str("[ ");
-	str += Token::name(iterSymbolNameCode);
-	str += " ]";
-	//cout << "added terminal " << t << endl;
-	int t = Token::encode(str.c_str());
-	parser->insertIterSymbolTerminal(iterSymbolNameCode, t);
-	rhs[0] = t;
-	parser->insertProduction(i->second, rhs, 0, emptyGather);
-      }
-  }
+  for (const auto& i : iterSymbols)
+    {
+      //
+      //	For each iter symbol f we create a special terminal
+      //	[ f ] which represents f^n for any positive interger n.
+      //
+      int iterSymbolNameCode = i.first;
+      string str("[ ");
+      str += Token::name(iterSymbolNameCode);
+      str += " ]";
+      int t = Token::encode(str.c_str());
+      parser->insertIterSymbolTerminal(iterSymbolNameCode, t);
+      rhs[0] = t;
+      parser->insertProduction(i.second, rhs, 0, emptyGather);
+    }
 }
 
 void
@@ -1531,7 +1558,7 @@ MixfixModule::makePolymorphProductions()
       if (nrItems > 0)
 	{
 	  mixfixRhs.resize(nrItems);
-	  underscores.contractTo(0);
+	  underscores.clear();
 	  for (int j = 0; j < nrItems; j++)
 	    {
 	      int t = p.symbolInfo.mixfixSyntax[j];
