@@ -2,7 +2,7 @@
 
     This file is part of the Maude 3 interpreter.
 
-    Copyright 1997-2020 SRI International, Menlo Park, CA 94025, USA.
+    Copyright 1997-2023 SRI International, Menlo Park, CA 94025, USA.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -58,6 +58,9 @@
 #include "variableSymbol.hh"
 #include "variableTerm.hh"
 
+//	ACU theory class definitions
+#include "ACU_Symbol.hh"
+
 //	S theory class definitions
 #include "S_Symbol.hh"
 #include "S_Term.hh"
@@ -97,23 +100,20 @@
 #include "mixfixModule.hh"
 #include "quotedIdentifierSymbol.hh"
 #include "quotedIdentifierTerm.hh"
+#include "objectConstructorSymbol.hh"
 #include "mixfixParser.hh"
 
 #define ROOT_NODE	(0)
 
-const IntSet&
-MixfixParser::getTokenSet()  // HACK
-{
-  return tokens;
-}
-
 MixfixParser::MixfixParser(MixfixModule& client,
 			   bool complexFlag,
 			   int componentNonTerminalBase,
-			   int nextNonTerminalCode)
+			   int nextNonTerminalCode,
+			   int nrTokensGuess)
   : client(client),
     complexParser(complexFlag),
     componentNonTerminalBase(componentNonTerminalBase),
+    tokenSet(nrTokensGuess),
     specialTerminals(Token::LAST_PROPERTY)
 {
   nextNonTerminal = nextNonTerminalCode;
@@ -142,7 +142,7 @@ MixfixParser::insertProduction(int lhs,
       int s = rhs[i];
       if (s < 0)
 	ntCount++;
-      productionRhs[i] = s < 0 ? s : tokens.insert(s);
+      productionRhs[i] = s < 0 ? s : tokenToIndex(s);
     }
 
 #ifndef NO_ASSERT
@@ -177,12 +177,12 @@ MixfixParser::insertBubbleProduction(int lhs,
 				     const Vector<int>& excluded,
 				     int bubbleSpecIndex)
 {
-  int left = (leftParenCode >= 0) ? tokens.insert(leftParenCode) : NONE;
-  int right = (rightParenCode >= 0) ? tokens.insert(rightParenCode) : NONE;
+  int left = (leftParenCode >= 0) ? tokenToIndex(leftParenCode) : NONE;
+  int right = (rightParenCode >= 0) ? tokenToIndex(rightParenCode) : NONE;
   int nrExcluded = excluded.length();
   Vector<int> excludedTerminals(nrExcluded);
   for (int i = 0; i < nrExcluded; i++)
-    excludedTerminals[i]= tokens.insert(excluded[i]);
+    excludedTerminals[i]= tokenToIndex(excluded[i]);
 
 #if PARSER_DEBUG
   cout << "insertProd(" << lhs <<
@@ -206,19 +206,19 @@ MixfixParser::insertBubbleProduction(int lhs,
 void
 MixfixParser::insertSpecialTerminal(int tokenProperty, int codeToUse)
 {
-  specialTerminals[tokenProperty] = tokens.insert(codeToUse);
+  specialTerminals[tokenProperty] = tokenToIndex(codeToUse);
 }
 
 void
 MixfixParser::insertVariableTerminal(int sortNameCode, int codeToUse)
 {
-  variableTerminals[sortNameCode] = tokens.insert(codeToUse);
+  variableTerminals[sortNameCode] = tokenToIndex(codeToUse);
 }
 
 void
 MixfixParser::insertIterSymbolTerminal(int iterSymbolNameCode, int codeToUse)
 {
-  iterSymbolTerminals[iterSymbolNameCode] = tokens.insert(codeToUse);
+  iterSymbolTerminals[iterSymbolNameCode] = tokenToIndex(codeToUse);
 }
 
 int
@@ -230,7 +230,7 @@ MixfixParser::translateSpecialToken(int code)
       int varName;
       int sortName;
       Token::split(code, varName, sortName);
-      IntMap::const_iterator i = variableTerminals.find(sortName);
+      auto i = variableTerminals.find(sortName);
       if (i != variableTerminals.end())
 	return i->second;
     }
@@ -239,14 +239,14 @@ MixfixParser::translateSpecialToken(int code)
       int opName;
       mpz_class dummy;
       Token::split(code, opName, dummy);
-      IntMap::const_iterator i = iterSymbolTerminals.find(opName);
+      auto i = iterSymbolTerminals.find(opName);
       if (i != iterSymbolTerminals.end())
 	return i->second;
     }
   else if (sp != NONE)
     return specialTerminals[sp];
   if (bubblesAllowed)
-    return tokens.cardinality();
+    return tokenSet.size();
   return NONE;
 }
 
@@ -265,8 +265,10 @@ MixfixParser::parseSentence(const Vector<Token>& original,
     {
       int j = begin + i;
       int code = original[j].code();
-      int terminal = tokens.int2Index(code);
-      if (terminal == NONE)
+
+      int terminal = NONE;
+      int entry = tokenSet.find(code);
+      if (entry == NONE)
 	{
 	  terminal = translateSpecialToken(code);
 	  if (terminal == NONE)
@@ -275,6 +277,8 @@ MixfixParser::parseSentence(const Vector<Token>& original,
 	      return -1;
 	    }
 	}
+      else
+	terminal = entry;
       sentence[i] = terminal;
     }
 
@@ -734,6 +738,42 @@ MixfixParser::makeTerm(int node)
 	t = symbol->makeTerm(args);
 	break;
       }
+    case MAKE_OBJECT_WITH_EMPTY_ATTRIBUTE_SET:
+      {
+	ObjectConstructorSymbol* symbol = safeCast(ObjectConstructorSymbol*, client.getSymbols()[a.data]);
+	if (client.getStatus() == Module::SIGNATURE_CLOSED)
+	  {
+	    //
+	    //	We haven't completed fix-ups, so identity need not exist.
+	    //
+	    int pos = currentOffset + parser.getFirstPosition(node);
+	    IssueWarning(LineNumber((*currentSentence)[pos].lineNumber()) <<
+			 ": empty attribute set syntax is not allowed in operator declarations.");
+	    client.markAsBad();
+	    //
+	    //	Because the sentence parsed ok, none of the code that calls makeTerm() is prepared
+	    //	to deal with failure so we need make a fake subterm so that the module can be deleted
+	    //	without dereferencing null.
+	    //
+	    Sort* sort = symbol->getRangeSort();
+	    VariableSymbol* vs = safeCast(VariableSymbol*, client.instantiateVariable(sort));
+	    t = new VariableTerm(vs, Token::encode("error"));
+	  }
+	else
+	  {
+	    //
+	    //	We shouldn't be parsing statements or commands if the identity could not be set.
+	    //
+	    ACU_Symbol* attributeSetSymbol = symbol->getAttributeSetSymbol();
+	    Term* identity = attributeSetSymbol->getIdentity();
+	    Assert(identity != 0, "null identity");
+	    for (int i = 0; i < 2; i++)
+	      args.append(makeTerm(parser.getChild(node, i)));
+	    args.append(identity->deepCopy());
+	    t = symbol->makeTerm(args);
+	  }
+	break;
+      }
     case MAKE_NATURAL:
       {
 	SuccSymbol* symbol = safeCast(SuccSymbol*, client.getSymbols()[a.data]);
@@ -1013,6 +1053,18 @@ MixfixParser::makeAttributePart(int node,
 	    flags.setFlags(NARROWING);
 	    break;
 	  }
+	case MAKE_DNT_ATTRIBUTE:
+	  {
+	    if (client.isObjectOriented())
+	      flags.setFlags(DNT);
+	    else
+	      {
+		int pos = currentOffset + parser.getFirstPosition(node);
+		IssueWarning(LineNumber((*currentSentence)[pos].lineNumber()) <<
+			     ": dnt attribute is only allowed in omods/oths.");
+	      }
+	    break;
+	  }
 	case MAKE_PRINT_ATTRIBUTE:
 	  {
 	    flags.setFlags(PRINT);
@@ -1137,6 +1189,7 @@ MixfixParser::makeStatementPart(int node,
 	client.insertSortConstraint(sc);
 	if (metadata != NONE)
 	  client.insertMetadata(MixfixModule::MEMB_AX, sc, metadata);
+	client.handleSortConstraint(sc, flags.getFlag(DNT));
 	if (flags.getFlag(PRINT))
 	  client.insertPrintAttribute(MixfixModule::MEMB_AX, sc, printNames, printSorts);
 	break;
@@ -1163,6 +1216,7 @@ MixfixParser::makeStatementPart(int node,
 	client.insertEquation(eq);
 	if (metadata != NONE)
 	  client.insertMetadata(MixfixModule::EQUATION, eq, metadata);
+	client.handleEquation(eq, flags.getFlag(DNT));
 	if (flags.getFlag(PRINT))
 	  client.insertPrintAttribute(MixfixModule::EQUATION, eq, printNames, printSorts);
 	break;
@@ -1192,6 +1246,7 @@ MixfixParser::makeStatementPart(int node,
 	client.insertRule(rl);
 	if (metadata != NONE)
 	  client.insertMetadata(MixfixModule::RULE, rl, metadata);
+	client.handleRule(rl, flags.getFlag(DNT));
 	if (flags.getFlag(PRINT))
 	  client.insertPrintAttribute(MixfixModule::RULE, rl, printNames, printSorts);
 	break;
