@@ -2,7 +2,7 @@
 
     This file is part of the Maude 3 interpreter.
 
-    Copyright 1997-2010 SRI International, Menlo Park, CA 94025, USA.
+    Copyright 1997-2024 SRI International, Menlo Park, CA 94025, USA.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -36,12 +36,12 @@
 
 //	our stuff
 #include "auxProperty.cc"
+#include "latexToken.cc"
 
 StringTable Token::stringTable;
 Vector<int> Token::specialProperties;
 Vector<int> Token::auxProperties;
-char* Token::buffer = 0;
-int Token::bufferLength = 0;
+Vector<char> Token::buffer;
 
 ostream&
 operator<<(ostream& s, const Token& token)
@@ -57,6 +57,13 @@ operator<<(ostream& s, const Vector<Token>& tokens)
   return s;
 }
 
+ostream&
+operator<<(ostream& s, const Token* token)
+{
+  s << LineNumber(token->lineNumber());
+  return s;
+}
+
 void
 Token::printTokens(ostream& s,
 		   const Vector<Token>& tokens,
@@ -110,18 +117,6 @@ Token::printTokenVector(ostream& s,
     }
 }
 
-void
-Token::reallocateBuffer(int length)
-{
-  length *= 2;  // to avoid piecemeal expansion
-  char* newBuffer = new char[length];
-  // FIXME Undefined behaviour when buffer is null and that happens
-  (void) memcpy(newBuffer, buffer, bufferLength);
-  delete [] buffer;
-  buffer = newBuffer;
-  bufferLength = length;
-}
-
 int
 Token::makeParameterInstanceName(int parameterCode, int originalCode)
 {
@@ -131,33 +126,6 @@ Token::makeParameterInstanceName(int parameterCode, int originalCode)
   return encode(newName.c_str());
 }
 
-/*
-bool
-Token::splitParameterInstanceName(int code, int& parameterName, int& baseName)
-{
-  //
-  //	Checks for a parameter instance name of the form:
-  //	  parameterName$baseName
-  //	and if so, splits it and returns true. Otherwise returns false.
-  //
-  const char* n = name(code);
-  for (const char* p = n; *p; ++p)
-    {
-      if (*p == '$' && p != n && *(p + 1) != 0)  // first $ not at one end
-	{
-	  size_t parameterLength = p - n;
-	  char* t = new char[parameterLength + 1];
-	  strncpy(t, n, parameterLength);
-	  t[parameterLength] = '\0';
-	  parameterName = encode(t);
-	  baseName = encode(p + 1);
-	  return true;
-	}
-    }
-  return false;
-}
-*/
-
 void
 Token::fixUp(const char* tokenString, int& lineNumber)
 {
@@ -167,14 +135,14 @@ Token::fixUp(const char* tokenString, int& lineNumber)
   //	We also convert \t characters to spaces.
   //
   int nrBackslashNewlineCombos = 0;
-  int j = 0;
+  buffer.clear();
   for (int i = 0;; i++)
     {
       char c = tokenString[i];
       if (c == '\\' && tokenString[i + 1] == '\n')
 	{
 	  //
-	  //	Fix up \ newline case.
+	  //	Skip over \ newline pair.
 	  //
 	  ++i;
 	  ++nrBackslashNewlineCombos;
@@ -187,14 +155,12 @@ Token::fixUp(const char* tokenString, int& lineNumber)
 		   ": tab character in string literal - replacing it with space");
 	      c = ' ';
 	    }
-	  bufferExpandTo(j + 1);
-	  buffer[j] = c;
-	  ++j;
+	  buffer.push_back(c);
 	  if (c == '\0')
 	    break;
 	}
     } 
-  codeNr = encode(buffer);
+  codeNr = encode(buffer.data());
   lineNr = lineNumber;
   lineNumber += nrBackslashNewlineCombos;
 }
@@ -205,36 +171,32 @@ Token::fixUp(const char* tokenString)
   //
   //	Remove \ newline sequences.
   //
-  int nrBackslashNewlineCombos = 0;
-  int j = 0;
+  buffer.clear();
   for (int i = 0;; i++)
     {
       char c = tokenString[i];
       if (c == '\\' && tokenString[i + 1] == '\n')
 	{
 	  //
-	  //	Fix up \ newline case.
+	  //	Skip over \ newline pair.
 	  //
 	  ++i;
-	  ++nrBackslashNewlineCombos;
 	}
       else
 	{
-	  bufferExpandTo(j + 1);
-	  buffer[j] = c;
-	  ++j;
+	  buffer.push_back(c);
 	  if (c == '\0')
 	    break;
 	}
-    } 
-  return encode(buffer);
+    }
+  return encode(buffer.data());
 }
 
 void
 Token::dropChar(const Token& original)
 {
   string truncated(stringTable.name(original.codeNr));
-  truncated.resize(truncated.size() - 1);
+  truncated.pop_back();
   codeNr = encode(truncated.c_str());
   lineNr = original.lineNr;
 }
@@ -264,10 +226,141 @@ Token::getInt(int& value) const
   return pointer != str && *pointer == '\0';
 }
 
+pair<string, bool>
+Token::makePrettyOpName(int prefixNameCode, int situations)
+{
+  //
+  //	We try to prettify a single token operator name by removing backquotes that
+  //	join special characters or indicate whitespace. This can cause ambiguities in
+  //	a number of situations and we check for these and return the problematic flag
+  //	indicating that the caller needs to wrap the result in parentheses.
+  //
+  //	Unbalanced parentheses are always an issue and blocks prettification.
+  //	Having a result that is already wrapped in parentheses is always problematic.
+  //
+  //	If we don't change the original, flag this by returning the empty string.
+  //
+  const char* original = stringTable.name(prefixNameCode);
+  DebugInfo("original = " << original);
+  int sp = specialProperties[prefixNameCode];
+  if (sp != NONE && sp != CONTAINS_COLON && sp != ENDS_IN_COLON && sp != ITER_SYMBOL)
+    return {"", false};  // looks like something that doen't need prettifying and is unproblematic
+
+  bool problematic = false;
+  //
+  //	If the name starts with "term" and it is not connected to the succeeding characters, we have a parsing issue in views.
+  //
+  if ((situations & EXPOSED_TERM) && original[0] == 't' && original[1] == 'e' && original[2] == 'r' && original[3] == 'm')
+    {
+      char c = original[4];
+      if (c == '`' || c == '\0')
+	problematic = true;
+    }
+  //
+  //	Prettify by removing backquotes, checking if we expose problematic tokens outside of parentheses.
+  //
+  bool prettified = false;
+  string result;
+  int parenDepth = 0;
+  bool disconnected = true;
+  for (const char* p = original; *p; ++p)
+    {
+      char c = *p;
+      if (c == '`')
+	{
+	  char c = *(p + 1);
+	  Assert(c != '\0', "can't end token with backquote");
+	  if (c == '(')
+	    ++parenDepth;
+	  else if (c == ')')
+	    {
+	      --parenDepth;
+	      if (parenDepth < 0)
+		return {"", false};  // don't prettify unbalanced parentheses; single token will not be problematic
+	    }
+	  if (specialChar(c))
+	    {
+	      //
+	      //	Handle backquoted special character.
+	      //
+	      result += c;
+	      ++p;
+	      if (!problematic && parenDepth == 0)
+		{
+		  //
+		  //	Special characters are always disconnected, so
+		  //	  , [
+		  //	outside of parentheses can be problematic.
+		  //
+		  if (((situations & EXPOSED_COMMA) && c == ',') ||
+		      ((situations & EXPOSED_LEFT_BRACKET) && c == '['))
+		    problematic = true;
+		}
+	    }
+	  else
+	    result += ' ';  // white space
+	  //
+	  //	Either we removed a backquote or turned the backquote into a space.
+	  //	Both count as prettified.
+	  //
+	  prettified = true;
+	  if (!problematic && (situations & MULTIPLE_TOKENS))
+	    {
+	      //
+	      //	Check if we will have multiple tokens.
+	      //
+	      if (result.length() > 1 || *(p + 1) != '\0')
+		problematic = true;
+	    }
+	  disconnected = true;
+	}
+      else
+	{
+	  result += c;
+	  if (!problematic && disconnected && parenDepth == 0)
+	    {
+	      //
+	      //	The tokens
+	      //	  . to
+	      //	appearing outside of parentheses and not connected to preceeding or succeeding
+	      //	characters can be problematic. Disconnected colons can be problematic even inside
+	      //	parentheses
+	      //
+	      if ((c == ':' && (((situations & EXPOSED_COLON) && parenDepth == 0) || (situations & BARE_COLON))) ||
+		  (c == '.' && (situations & EXPOSED_DOT) && parenDepth == 0))
+		{
+		  char c = *(p + 1);
+		  if (c == '`' || c == '\0')
+		    problematic = true;
+		}
+	      else if ((situations & EXPOSED_TO) && parenDepth == 0 && c == 't' && *(p + 1) == 'o')
+		{
+		  char c = *(p + 2);
+		  if (c == '`' || c == '\0')
+		    problematic = true;
+		}
+	    }
+	  disconnected = false;
+	}
+    }
+  if (parenDepth != 0)
+    return {"", false};  // don't prettify unbalanced parentheses; single token will not be problematic
+  //
+  //	If the first and last characters of the prettified name are ( and ), they will be eliminated
+  //	during parsing which is always problematic.
+  //
+  if (!problematic && result.front() == '(' && result.back() == ')')
+    problematic = true;
+  DebugInfo("result = " << result << " problematic = " << problematic);
+  if (prettified)
+    return {result, problematic};
+  else
+    return {"", problematic};
+}
+
 int
 Token::extractMixfix(int prefixNameCode, Vector<int>& mixfixSyntax)
 {
-  
   int sp = specialProperties[prefixNameCode];
   if (sp != NONE && sp != CONTAINS_COLON && sp != ENDS_IN_COLON && sp != ITER_SYMBOL)
     return 0;  // regular string literals exit here
@@ -288,6 +381,7 @@ Token::extractMixfix(int prefixNameCode, Vector<int>& mixfixSyntax)
   bool stringMode = false;
   bool seenBS = false;
   string token(name, p - name);
+  int parenDepth = 0;
   for (;; p++)
     {
       char c = *p;
@@ -328,6 +422,17 @@ Token::extractMixfix(int prefixNameCode, Vector<int>& mixfixSyntax)
 	  //
 	  //	Special character: terminate current token and add special character as a token.
 	  //
+	  if (c == '(')
+	    ++parenDepth;
+	  else if (c == ')')
+	    {
+	      --parenDepth;
+	      if (parenDepth < 0)
+		{
+		  mixfixSyntax.clear();
+		  return PAREN_MISMATCH;
+		}
+	    }
 	  if (!token.empty())
 	    mixfixSyntax.append(encode(token.c_str()));
 	  token = c;
@@ -347,30 +452,39 @@ Token::extractMixfix(int prefixNameCode, Vector<int>& mixfixSyntax)
       DebugAdvisory("deleting mixfixSyntax for " << name);
       mixfixSyntax.clear();
     }
+  //
+  //	If parentheses are mismatched, we ignore mixfix syntax since it won't be parsable.
+  //
+  if (parenDepth != 0)
+    {
+      mixfixSyntax.clear();
+      return PAREN_MISMATCH;
+    }
   return nrUnderscores;
 }
 
 void
 Token::checkForSpecialProperty(const char* tokenString)
 {
-  int tokenNr = specialProperties.length();
-  specialProperties.expandBy(1);
-  specialProperties[tokenNr] = NONE;
-  auxProperties.expandBy(1);
-  auxProperties[tokenNr] = computeAuxProperty(tokenString);
+  specialProperties.push_back(computeSpecialProperty(tokenString));
+  auxProperties.push_back(computeAuxProperty(tokenString));
+}
 
+int
+Token::computeSpecialProperty(const char* tokenString)
+{
   const char* p = tokenString;
   if (*p == 0)
-    return;  // handle null strings correctly
+    return NONE;  // handle null strings correctly
   if (*p == '\'')
-    {
-      specialProperties[tokenNr] = QUOTED_IDENTIFIER;
-      return;
-    }
+    return QUOTED_IDENTIFIER;  // anything starting with a ' is a quoted identifier
   if (*p == '"')
     {
+      //
+      //	Check for string.
+      //
       bool seenBackslash = false;
-      for (p++; *p; p++)
+      for (++p; *p; ++p)
 	{
 	  char c = *p;
 	  if (c == '\\')
@@ -380,16 +494,23 @@ Token::checkForSpecialProperty(const char* tokenString)
 	      if (c == '"' && !seenBackslash)
 		{
 		  if (*(p + 1) == '\0')
-		    specialProperties[tokenNr] = STRING;
-		  return;
+		    return STRING;
+		  break;  // can't be a STRING
 		}
 	      seenBackslash = false;
 	    }
 	}
-      return;
     }
+  //
+  //	Anthing that isn't as quoted identifer and ends in : gets the ENDS_IN_COLON property.
+  //
+  size_t len = strlen(tokenString);
+  if (len > 1 && tokenString[len - 1] == ':')
+    return ENDS_IN_COLON;
   {
-    size_t len = strlen(tokenString);
+    //
+    //	Tokens that look like f^123 get the ITER_SYMBOL property.
+    //
     for (size_t i = len - 1; i > 0; --i)
       {
 	char c = tokenString[i];
@@ -397,39 +518,34 @@ Token::checkForSpecialProperty(const char* tokenString)
 	  {
 	    if (i == len - 1 || tokenString[i + 1] == '0')
 	      break; 
-	    specialProperties[tokenNr] = ITER_SYMBOL;
-	    return;
+	    return ITER_SYMBOL;
 	  }
 	if (!isdigit(c))
 	  break;
       }
   }
-  for (++p; *p; ++p)
-    {
-      if (*p == ':')
-        specialProperties[tokenNr] = *(p + 1) ? CONTAINS_COLON : ENDS_IN_COLON;
-    }
-  if (specialProperties[tokenNr] != NONE)
-    return;
+  {
+    //
+    //	Anything that isn't a quoted identifer, string or iter, or ends in a colon, that contains at least one : other than
+    //	the first character get the CONTAINS_COLON property.
+    //
+    for (++p; *p; ++p)  // don't count first character
+      {
+	if (*p == ':')
+	  return CONTAINS_COLON;
+      }
+  }
   if (looksLikeFloat(tokenString))
-    {
-      specialProperties[tokenNr] = FLOAT;
-      return;
-    }
-
-  mpz_class i;
-  int error = mpz_set_str(i.get_mpz_t(), tokenString, 10);
-  if (error == 0)
-    {
-      specialProperties[tokenNr] = (i == 0) ? ZERO : ((i < 0) ? SMALL_NEG : SMALL_NAT);
-      return;
-    }
-
+    return FLOAT;
+  {
+    mpz_class i;
+    int error = mpz_set_str(i.get_mpz_t(), tokenString, 10);
+    if (error == 0)
+      return (i == 0) ? ZERO : ((i < 0) ? SMALL_NEG : SMALL_NAT);
+  }
   if (looksLikeRational(tokenString))
-    {
-      specialProperties[tokenNr] = RATIONAL;
-      return;
-    }
+    return RATIONAL;
+  return NONE;
 }
 
 bool
@@ -481,23 +597,23 @@ Token::looksLikeRational(const char* s)
 bool
 Token::split(int code, int& prefix, int& suffix)
 {
+  buffer.clear();
   const char* p = stringTable.name(code);
-  size_t len = strlen(p);
-  char* t = new char[len + 1];
-  strcpy(t, p);
-  for (size_t i = len - 1; i > 0; --i)
+  Index len = strlen(p);
+  for (Index i = len - 1; i > 0; --i)  // don't consider ':' or '.' in p[0]
     {
-      char c = t[i];
+      char c = p[i];
       if (c == ':' || c == '.')
 	{
-	  t[i] = '\0';
-	  prefix = encode(t);
-	  suffix = (i == len - 1) ? NONE : encode(t + i + 1);
-	  delete [] t;
+	  suffix = (i == len - 1) ? NONE : encode(p + i + 1);
+	  buffer.resize(i + 1);
+	  for (Index j = 0; j != i; ++j)
+	    buffer[j] = p[j];
+	  buffer[i] = '\0';
+	  prefix = encode(buffer.data());
 	  return true;
 	}
     }
-  delete [] t;
   return false;
 }
 
@@ -533,38 +649,42 @@ Token::splitKind(int code, Vector<int>& codes)
 {
   codes.clear();
   const char* p = stringTable.name(code);
-  size_t len = strlen(p);
-  char* t = new char[len + 1];
-  p = strcpy(t, p);
+  buffer.resize(strlen(p) + 1);
+  p = strcpy(buffer.data(), p);
+  //
+  //	A kind must start with `[
+  //
   if (*p++ =='`' && *p++ == '[')
     {
       for(;;)
 	{
 	  bool dummy;
-	  char *p2 = const_cast<char*>(skipSortName(p, dummy));
-	  if (p2 != 0 && *p2 == '`')
+	  const char* p2 = skipSortName(p, dummy);
+	  if (p2 != nullptr && *p2 == '`')
 	    {
-	      *p2 = 0;
-	      codes.append(encode(p));
+	      //
+	      //	Terminator is a backquoted special character. Only `, and `] are legal within a kind.
+	      //
+	      Index length = p2 - p;
+	      for (Index i = 0; i < length; ++i)
+		buffer[i] = p[i];
+	      buffer[length] = '\0';
+	      codes.append(encode(buffer.data()));
 	      p = p2 + 1;
 	      switch (*p++)
 		{
 		case ']':
-		  {
-		    if (*p == '\0')
-		      {
-			delete [] t;
-			return true;
-		      }
-		  }
+		  return *p == '\0';  // we're good as long as there were no surplus characters
 		case ',':
-		  continue;
+		  continue;  // look for another sort name
 		}
 	    }
-	  break;
+	  //
+	  //	Bad sort name or illegal terminator; return false.
+	  //
+	  return false;
 	}
     }
-  delete [] t;
   return false;
 }
 
@@ -621,103 +741,6 @@ Token::doubleToCode(double d)
   return code;
 }
 
-/*
-Rope
-Token::codeToRope(int code)
-{
-  Rope result;
-  bool seenBackslash = false;
-  for (const char* p = stringTable.name(code) + 1; *p; p++)
-    {
-      char c = *p;
-      switch (c)
-	{
-	case '\\':
-	  {
-	    if (!seenBackslash)
-	      {
-		seenBackslash = true;
-		continue;
-	      }
-	    break;
-	  }
-	case '"':
-	  {
-	    if (!seenBackslash)
-	      return result;
-	    break;
-	  }
-	case 'a':
-	  {
-	    if (seenBackslash)
-	      c = '\a';
-	    break;
-	  }
-	case 'b':
-	  {
-	    if (seenBackslash)
-	      c = '\b';
-	    break;
-	  }
-	case 'f':
-	  {
-	    if (seenBackslash)
-	      c = '\f';
-	    break;
-	  }
-	case 'n':
-	  {
-	    if (seenBackslash)
-	      c = '\n';
-	    break;
-	  }
-	case 'r':
-	  {
-	    if (seenBackslash)
-	      c = '\r';
-	    break;
-	  }
-	case 't':
-	  {
-	    if (seenBackslash)
-	      c = '\t';
-	    break;
-	  }
-	case 'v':
-	  {
-	    if (seenBackslash)
-	      c = '\v';
-	    break;
-	  }
-	default:
-	  {
-	    if (seenBackslash && isdigit(c) && c != '8' && c != '9')
-	      {
-		int i = c - '0';
-	        c = *(p + 1);
-		if (isdigit(c) && c != '8' && c != '9')
-		  {
-		    ++p;
-		    i = 8 * i + c - '0';
-		    c = *(p + 1);
-		    if (isdigit(c) && c != '8' && c != '9')
-		      {
-			++p;
-			i = 8 * i + c - '0';
-		      }
-		  }
-		c = i;
-	      }
-	  }
-	}
-      result += c;
-      seenBackslash = false;
-    }
-  CantHappen("bad end to string");
-  return result;
-}
-*/
-
 Rope
 Token::stringToRope(const char* string)
 {
@@ -941,7 +964,7 @@ Token::ropeToPrefixNameCode(const Rope& r)
 int
 Token::bubbleToPrefixNameCode(const Vector<Token>& opBubble)
 {
-  int nrTokens = opBubble.length();
+  int nrTokens = opBubble.size();
   if (nrTokens == 1)
     {
       int code = opBubble[0].codeNr;
@@ -950,8 +973,8 @@ Token::bubbleToPrefixNameCode(const Vector<Token>& opBubble)
       if (!specialChar(stringTable.name(code)[0]))
         return code;
     }
-  int pos = 0;
   bool needBQ = false;
+  buffer.clear();
   for (int i = 0; i < nrTokens; i++)
     {
       const char* name = stringTable.name(opBubble[i].codeNr);
@@ -961,21 +984,16 @@ Token::bubbleToPrefixNameCode(const Vector<Token>& opBubble)
       else if (c == '_' || c == '`')
         needBQ = false;
       if (needBQ)
-        {
-          bufferExpandTo(pos + 1);
-          buffer[pos++] = '`';
-        }
+	buffer.push_back('`');
       while (*name != '\0')
-        {
-          c = *name++;
-          bufferExpandTo(pos + 1);
-          buffer[pos++] = c;
-        }
+	{
+	  c = *name++;
+	  buffer.push_back(c);
+	}
       needBQ = !(specialChar(c) || c == '_');
     }
-  bufferExpandTo(pos + 1);
-  buffer[pos] = '\0';
-  return encode(buffer);
+  buffer.push_back('\0');
+  return encode(buffer.data());
 }
 
 void
