@@ -2,7 +2,7 @@
 
     This file is part of the Maude 3 interpreter.
 
-    Copyright 1997-2021 SRI International, Menlo Park, CA 94025, USA.
+    Copyright 1997-2023 SRI International, Menlo Park, CA 94025, USA.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -425,7 +425,8 @@ StreamManagerSymbol::interruptHandler(int)
   //	We expect that we will have an incomplete line bacause we're not
   //	using Tecla so we force a newline before exiting.
   //
-  ::write(STDIN_FILENO, "\n", 1);
+  if (::write(STDIN_FILENO, "\n", 1))  // exiting the subprocess so we *really* don't care about the result
+    ;
   exit(0);
 }
 
